#!/bin/bash

# test is we are running on Ubuntu
if [ $(uname -v | cut -d '-' -f 2 | awk '{print $1}') != "Ubuntu" ];
then
  echo
  echo This script currently only works on Ubuntu
  echo
  exit
fi

if [ ! -f ./virtualhost.conf  ]; then
    echo "One of the configuration files are missing"
    exit 1
fi

if [ $# -eq 0 ]
  then
    echo
    echo $(tput setaf 1)
    echo This script will install Zabbix under Apache
    echo If Apache can\'t be found then it will be installed first
    echo
    echo It needs SUDO privileges so will ask for them where necessary
    echo
    echo "Usage: ./InstallZabbixServer.sh domain_name (without http://, https://)"
    echo
    echo "Note: Make sure that this domain name is already pointing to the server"
    echo "      and a CNAME is set up for www"
    echo
    echo "Your site will be available at https://(www.)[domain_name]"
    echo
    echo $(tput setaf 7)
    exit 1
fi

sitename=$1

if [[ "$sitename" == *"http://"* ]]; then
  echo "Usage: ./InstallZabbixServer.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"https://"* ]]; then
  echo "Usage: ./InstallZabbixServer.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"HTTP://"* ]]; then
  echo "Usage: ./InstallZabbixServer.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

if [[ "$sitename" == *"HTTPS://"* ]]; then
  echo "Usage: ./InstallZabbixServer.sh domain_name (without http\:\/\/, https\:\/\/ or www\.)"
  exit 1
fi

# Install some needed utilities if not already installed
sudo apt install lsb-release ca-certificates apt-transport-https software-properties-common language-pack-en-base unzip -y

# Install PHP
phpver=8.1

sudo add-apt-repository ppa:ondrej/php -y
sudo apt update
sudo apt install php$phpver-{common,cli,bz2,zip,curl,intl,mysql,snmp,memcached,imagick,gd,imap,ldap,soap,tidy,xml,gmp,pspell,mbstring,opcache,fpm,ssh2,imap,redis,apcu,mcrypt} -y
sudo apt install ghostscript imagemagick -y

# Configure PHP
sudo sed -i -e '/^\(;\|\)listen.mode = 0660/s/^.*$/listen.mode = 0660/' /etc/php/$phpver/fpm/pool.d/www.conf
sudo sed -i -e '/^\(;\|\)output_buffering = 4096/s/^.*$/output_buffering = Off/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)expose_php =/s/^.*$/expose_php = Off/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)max_execution_time =/s/^.*$/max_execution_time = 300/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)max_input_time =/s/^.*$/max_input_time = 300/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)max_input_vars =/s/^.*$/max_input_vars = 1000/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)memory_limit =/s/^.*$/memory_limit = 512M/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)post_max_size =/s/^.*$/post_max_size = 8M/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)upload_max_filesize =/s/^.*$/upload_max_filesize = 4M/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)allow_url_fopen =/s/^.*$/allow_url_fopen = Off/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)date.timezone =/s/^.*$/date.timezone = Europe\/London/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.enable=/s/^.*$/opcache.enable=1/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.memory_consumption=/s/^.*$/opcache.memory_consumption=128/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.max_accelerated_files=/s/^.*$/opcache.max_accelerated_files=10000/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.revalidate_freq=/s/^.*$/opcache.revalidate_freq=2/' /etc/php/$phpver/fpm/php.ini
sudo sed -i -e '/^\(;\|\)opcache.save_comments=/s/^.*$/opcache.save_comments=1/' /etc/php/$phpver/fpm/php.ini
sudo systemctl restart php$phpver-fpm

uver=$(lsb_release -sr)

# Check if Apache is installed and install it if not
if ! command -v apache2 &> /dev/null
then
  sudo apt-get install apache2 apache2-utils libapache2-mod-fcgid
  sudo a2enmod mime http2 rewrite deflate expires headers ssl
  sudo a2enmod proxy_fcgi setenvif mpm_event
  sudo a2enconf php8.1-fpm
  sudo sed -i -e '/^\(;\|\)ServerTokens OS/s/^.*$/ServerTokens Prod/' /etc/apache2/conf-enabled/security.conf
  sudo sed -i -e '/^\(;\|\)ServerSignature On/s/^.*$/ServerSignature Off/' /etc/apache2/conf-enabled/security.conf
  echo "Header unset ETag" | sudo tee -a /etc/apache2/conf-enabled/security.conf
  echo "FileETag None" | sudo tee -a /etc/apache2/conf-enabled/security.conf

  sudo sed -i -e '/^\(;\|\)Timeout 300/s/^.*$/Timeout 120/' /etc/apache2/apache2.conf
  sudo sed -i -e '/^\(;\|\)KeepAliveTimeout 5/s/^.*$/KeepAliveTimeout 10/' /etc/apache2/apache2.conf
  echo "ProxyRequests Off" | sudo tee -a /etc/apache2/apache2.conf
  echo "ProxyVia Off" | sudo tee -a /etc/apache2/apache2.conf
  echo '<Proxy "*">' | sudo tee -a /etc/apache2/apache2.conf
  echo "Order Deny,Allow" | sudo tee -a /etc/apache2/apache2.conf
  echo "Deny from all" | sudo tee -a /etc/apache2/apache2.conf
  echo "Allow from 192.168.0" | sudo tee -a /etc/apache2/apache2.conf
  echo "</Proxy>" | sudo tee -a /etc/apache2/apache2.conf
  sudo a2dissite 000-default.conf
  sudo rm /var/www/html/*
  sudo systemctl restart apache2
fi

# Check if MariaDB is installed and install it if not
if ! command -v mysql &> /dev/null
then
  sudo apt install mariadb-server -y
  sudo mysql_secure_installation
  sudo sed -i -e '/^\(;\|\)bind-address            = 127.0.0.1/s/^.*$/bind-address            = 127.0.0.1\nlocal_infile = 0/' /etc/mysql/mariadb.conf.d/50-server.cnf
  sudo systemctl restart mariadb
fi

# Create database
databasename=$(openssl rand -hex 2)
databaseuser=$(openssl rand -hex 2)
databasepass=$(openssl rand -hex 8)

sudo mysql -e "CREATE DATABASE $databasename CHARACTER SET utf8mb4"
sudo mysql -e "CREATE USER '$databaseuser'@'localhost' IDENTIFIED BY '$databasepass'"
sudo mysql -e "GRANT ALL PRIVILEGES ON $databasename.* TO '$databaseuser'@'localhost'"
sudo mysql -e "set global log_bin_trust_function_creators = 1"
sudo mysql -e "FLUSH PRIVILEGES"

# Install Zabbix server
cd ~
# visit https://www.zabbix.com/download for other versions
wget https://repo.zabbix.com/zabbix/6.4/ubuntu/pool/main/z/zabbix-release/zabbix-release_6.4-1+ubuntu22.04_all.deb
sudo dpkg -i zabbix-release_6.4-1+ubuntu22.04_all.deb
sudo apt update
sudo apt install zabbix-server-mysql zabbix-frontend-php zabbix-apache-conf zabbix-sql-scripts
zcat /usr/share/zabbix-sql-scripts/mysql/server.sql.gz | mysql --default-character-set=utf8mb4 -u$databaseuser -p $databasepass
sudo mysql -e "set global log_bin_trust_function_creators = 0"
sudo sed -i -e "/^\(;\|\)DBPassword=/s/^.*$/DBPassword=$databasepass/" /etc/zabbix/zabbix_server.conf

# Install Zabbix agent
sudo apt install zabbix-agent2
sudo sed -i -e "/^\(;\|\)Hostname=/s/^.*$/Hostname=$(hostname)/" /etc/zabbix/zabbix_agent2.conf
sudo systemctl restart zabbix-server zabbix-agent2 apache2
sudo systemctl enable zabbix-server zabbix-agent2 apache2

# Setup virtual host
sudo sed -i -e "/^\(;\|\)ServerName/s/^.*$/ServerName $sitename/" ./virtualhost.conf
sudo sed -i -e "/^\(;\|\)ServerAdmin/s/^.*$/ServerAdmin admin@$sitename/" ./virtualhost.conf
sudo sed -i -e "/^\(;\|\)DocumentRoot \/var\/www\/html/s/^.*$/\/usr\/share\/zabbix/" ./virtualhost.conf
sudo mv ./virtualhost.conf /etc/apache2/sites-available/zabbix.conf
sudo a2ensite zabbix
sudo systemctl reload apache2

# Create a TLS certificate for the site
if ! command -v certbot &> /dev/null
then
  sudo apt-get install snapd
  sudo snap install core; sudo snap refresh core
  sudo snap install --classic certbot
  sudo ln -s /snap/bin/certbot /usr/bin/certbot
fi

sudo certbot -d $sitename --apache

# Finished
echo.
echo Installation complete
echo
echo You should now visit $sitename/setup.php in your browser to finish the installation
echo
echo You will need, and should make a record of, the following details
echo
echo Site name: $sitename
echo
echo Database name    : $databasename
echo Database user    : $databaseuser
echo Database password: $databasepass
echo